/*
 * rottest.c --- test program for alti-aliased rotation
 *
 * This file is gift-ware.  This file is given to you freely
 * as a gift.  You may use, modify, redistribute, and generally hack
 * it about in any way you like, and you do not have to give anyone
 * anything in return.
 *
 * I do not accept any responsibility for any effects, adverse or
 * otherwise, that this code may have on just about anything that
 * you can think of.  Use it at your own risk.
 *
 * Copyright (C) 1998, 1999  Michael Bukin
 */

#include <allegro.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "aastr.h"

#define STEPS 16

static void
readesc (void)
{
  if ((readkey () >> 8) == KEY_ESC)
    exit (0);
}

static RGB_MAP rgb_table;

int
main (int _argc, char *_argv[])
{
  int i;
  int opt;
  int width = 640;
  int height = 480;
  int depth = 32;
  int picture_depth = 32;
  char *filename = "to.pcx";
  BITMAP *bmp;
  PALLETE pal;

  opterr = 0;
  while ((opt = getopt (_argc, _argv, "w:h:d:p:f:Z")) != EOF)
    {
      switch (opt)
	{
	case 'w':
	  width = atoi (optarg);
	  break;
	case 'h':
	  height = atoi (optarg);
	  break;
	case 'd':
	  depth = atoi (optarg);
	  break;
	case 'p':
	  picture_depth = atoi (optarg);
	  break;
	case 'f':
	  filename = optarg;
	  break;
	default:
	  fprintf (stderr, "%s: unrecognized option -- %c\n", _argv[0], optopt);
	case 'Z':
	  fprintf (stderr, "Usage: %s [OPTIONS]\n", _argv[0]);
	  fprintf (stderr,
		   "  -w NUM  Screen width.\n"
		   "  -h NUM  Screen height.\n"
		   "  -d NUM  Screen color depth.\n"
		   "  -p NUM  Picture color depth.\n"
		   "  -f FILE  Picture file.\n"
		   "\n"
		   "  -Z  This help\n");
	  exit (1);
	}
    }

  if (allegro_init () < 0)
    {
      fprintf (stderr, "%s: allegro_init failed\n", _argv[0]);
      exit (1);
    }
  if (install_timer () < 0)
    {
      fprintf (stderr, "%s: install_timer failed\n", _argv[0]);
      exit (1);
    }
  if (install_keyboard () < 0)
    {
      fprintf (stderr, "%s: install_keyboard failed\n", _argv[0]);
      exit (1);
    }

  set_color_depth (picture_depth);
  bmp = load_bitmap (filename, pal);
  if (bmp == 0)
    {
      allegro_exit ();
      fprintf (stderr, "%s: load_bitmap (%s) failed\n", _argv[0], filename);
      exit (1);
    }
  /* Validate pallete (create_rgb_table may fail if pallete is wrong).  */
  for (i = 0; i < 256; i++)
    {
      pal[i].r &= 0x3F;
      pal[i].g &= 0x3F;
      pal[i].b &= 0x3F;
    }

  set_color_depth (depth);
  if (set_gfx_mode (GFX_AUTODETECT, width, height, 0, 0) < 0)
    {
      fprintf (stderr, "%s: %s\n", _argv[0], allegro_error);
      exit (1);
    }

  set_pallete (pal);
  create_rgb_table (&rgb_table, pal, 0);
  rgb_map = &rgb_table;

  /* Show original bitmap.  */
  blit (bmp, screen, 0, 0, 0, 0, bmp->w, bmp->h);
  readesc ();

  /* Show the result of rotate_bitmap (0 degrees).  */
  aa_rotate_scaled_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, 0,
			   fdiv (itofix (SCREEN_W), itofix (bmp->w)),
			   fdiv (itofix (SCREEN_H), itofix (bmp->h)));
  readesc ();

  /* Show the result of rotate_bitmap (90 degrees).  */
  aa_rotate_scaled_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, itofix (64),
			   fdiv (itofix (SCREEN_H), itofix (bmp->w)),
			   fdiv (itofix (SCREEN_W), itofix (bmp->h)));
  readesc ();

  /* Masked rotation (expanding).  */
  clear (screen);
  while (!keypressed ())
    aa_rotate_scaled_sprite (screen, bmp,
			     random () % SCREEN_W, random () % SCREEN_H,
			     random () % itofix (512) - itofix (256),
			     itofix (random () % 16 - 8),
			     itofix (random () % 16 - 8));
  readesc ();

  /* Masked rotation (shrinking).  */
  clear (screen);
  while (!keypressed ())
    aa_rotate_scaled_sprite (screen, bmp,
			     random () % SCREEN_W, random () % SCREEN_H,
			     random () % itofix (512) - itofix (256),
			     random () % itofix (2) - itofix (1),
			     random () % itofix (2) - itofix (1));
  readesc ();

  /* Rotate and scale.  */
  {
    int i;
    fixed angle, scalex, scaley, dsx, dsy;

    /* Rotation and scaling.  */
    for (i = STEPS; i > 0; i--)
      {
	if (keypressed ())
	  break;
	scalex = fdiv (itofix ((i * SCREEN_W) / STEPS), itofix (bmp->w));
	scaley = fdiv (itofix ((i * SCREEN_H) / STEPS), itofix (bmp->h));
	angle = fdiv (itofix (i * 256), itofix (STEPS));
	aa_rotate_scaled_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, angle,
				 scalex, scaley);
      }
    readesc ();

    /* Rotation.  */
    angle = 0;
    while (!keypressed ())
      {
	angle += itofix (1);
	aa_rotate_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, angle);
      }
    readesc ();

    /* Rotation and scaling in sx.  */
    angle = 0;
    scalex = itofix (4);
    dsx = -itofix (1) / 16;
    while (!keypressed ())
      {
	angle += itofix (1);
	scalex += dsx;
	if ((scalex >= itofix (4)) || (scalex <= -itofix (4)))
	  dsx = -dsx;
	aa_rotate_scaled_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, angle,
				 scalex, itofix (1));
      }
    readesc ();

    /* Rotation and scaling in sy.  */
    angle = 0;
    scaley = itofix (4);
    dsy = -itofix (1) / 16;
    while (!keypressed ())
      {
	angle += itofix (1);
	scaley += dsy;
	if ((scaley >= itofix (4)) || (scaley <= -itofix (4)))
	  dsy = -dsy;
	aa_rotate_scaled_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, angle,
				 itofix (1), scaley);
      }
    readesc ();

    /* Rotation and scaling in both sx and sy.  */
    angle = 0;
    scalex = itofix (4);
    scaley = itofix (4);
    dsx = -itofix (1) / 16;
    dsy = -itofix (1) / 16;
    while (!keypressed ())
      {
	angle += itofix (1);
	scalex += dsx;
	if ((scalex >= itofix (4)) || (scalex <= -itofix (4)))
	  dsx = -dsx;
	scaley += dsy;
	if ((scaley >= itofix (4)) || (scaley <= -itofix (4)))
	  dsy = -dsy;
	aa_rotate_scaled_bitmap (bmp, screen, SCREEN_W / 2, SCREEN_H / 2, angle,
				 scalex, scaley);
      }
    readesc ();
  }

  destroy_bitmap (bmp);
  allegro_exit ();

  return 0;
}

/*
 * rottest.c ends here
 */
