/*
 * strtest.c --- test program for anti-aliased stretching
 *
 * This file is gift-ware.  This file is given to you freely
 * as a gift.  You may use, modify, redistribute, and generally hack
 * it about in any way you like, and you do not have to give anyone
 * anything in return.
 *
 * I do not accept any responsibility for any effects, adverse or
 * otherwise, that this code may have on just about anything that
 * you can think of.  Use it at your own risk.
 *
 * Copyright (C) 1998, 1999  Michael Bukin
 */

#include <allegro.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "aastr.h"

#define STEPS 16

static void
readesc (void)
{
  if ((readkey () >> 8) == KEY_ESC)
    exit (0);
}

static RGB_MAP rgb_table;

int
main (int _argc, char *_argv[])
{
  int i;
  int opt;
  int width = 640;
  int height = 480;
  int depth = 32;
  int picture_depth = 32;
  char *filename = "to.pcx";
  BITMAP *bmp;
  PALLETE pal;

  opterr = 0;
  while ((opt = getopt (_argc, _argv, "w:h:d:p:f:Z")) != EOF)
    {
      switch (opt)
	{
	case 'w':
	  width = atoi (optarg);
	  break;
	case 'h':
	  height = atoi (optarg);
	  break;
	case 'd':
	  depth = atoi (optarg);
	  break;
	case 'p':
	  picture_depth = atoi (optarg);
	  break;
	case 'f':
	  filename = optarg;
	  break;
	default:
	  fprintf (stderr, "%s: unrecognized option -- %c\n", _argv[0], optopt);
	case 'Z':
	  fprintf (stderr, "Usage: %s [OPTIONS]\n", _argv[0]);
	  fprintf (stderr,
		   "  -w NUM  Screen width.\n"
		   "  -h NUM  Screen height.\n"
		   "  -d NUM  Screen color depth.\n"
		   "  -p NUM  Picture color depth.\n"
		   "  -f FILE  Picture file.\n"
		   "\n"
		   "  -Z  This help\n");
	  exit (1);
	}
    }

  if (allegro_init () < 0)
    {
      fprintf (stderr, "%s: allegro_init failed\n", _argv[0]);
      exit (1);
    }
  if (install_timer () < 0)
    {
      fprintf (stderr, "%s: install_timer failed\n", _argv[0]);
      exit (1);
    }
  if (install_keyboard () < 0)
    {
      fprintf (stderr, "%s: install_keyboard failed\n", _argv[0]);
      exit (1);
    }

  set_color_depth (picture_depth);
  bmp = load_bitmap (filename, pal);
  if (bmp == 0)
    {
      allegro_exit ();
      fprintf (stderr, "%s: load_bitmap (%s) failed\n", _argv[0], filename);
      exit (1);
    }
  /* Validate pallete (create_rgb_table may fail if pallete is wrong).  */
  for (i = 0; i < 256; i++)
    {
      pal[i].r &= 0x3F;
      pal[i].g &= 0x3F;
      pal[i].b &= 0x3F;
    }

  set_color_depth (depth);
  if (set_gfx_mode (GFX_AUTODETECT, width, height, 0, 0) < 0)
    {
      fprintf (stderr, "%s: %s\n", _argv[0], allegro_error);
      exit (1);
    }

  set_pallete (pal);
  create_rgb_table (&rgb_table, pal, 0);
  rgb_map = &rgb_table;

  /* Show original bitmap.  */
  blit (bmp, screen, 0, 0, 0, 0, bmp->w, bmp->h);
  readesc ();

  /* Show the result of aa_stretch_blit.  */
  aa_stretch_blit (bmp, screen, 0, 0, bmp->w, bmp->h, 0, 0, SCREEN_W, SCREEN_H);
  readesc ();

  /* aa_stretch_blit in the middle of bitmap.  */
  aa_stretch_blit (bmp, screen, 0, 0, bmp->w, bmp->h, SCREEN_W / 4, SCREEN_H / 4,
		   SCREEN_W / 2, SCREEN_H / 2);
  readesc ();

  /* aa_stretch_blit from the middle of bitmap.  */
  aa_stretch_blit (bmp, screen, bmp->w / 4, bmp->h / 4, bmp->w / 2, bmp->h / 2,
		   0, 0, SCREEN_W, SCREEN_H);
  readesc ();

  /* Clipping by destination.  */
  aa_stretch_blit (bmp, screen, 0, 0, bmp->w, bmp->h,
		   -SCREEN_W, -SCREEN_H, 3 * SCREEN_W, 3 * SCREEN_H);
  readesc ();

  /* Masked stretch blit.  */
  clear (screen);
  while (!keypressed ())
    aa_stretch_sprite (screen, bmp,
		       random () % (2 * SCREEN_W) - SCREEN_W,
		       random () % (2 * SCREEN_H) - SCREEN_H,
		       random () % SCREEN_W, random () % SCREEN_H);
  readesc ();

  /* aa_stretch_blit for different stretching in different directions.  */
  {
    int i, x, y, w, h;

    x = 0;
    w = SCREEN_W;
    for (i = 0; i <= STEPS; i++)
      {
	if (keypressed ())
	  break;
	h = ((STEPS - i) * SCREEN_H) / STEPS;
	y = (SCREEN_H - h) / 2;
	aa_stretch_blit (bmp, screen, 0, 0, bmp->w, bmp->h, x, y, w, h);
      }
    y = 0;
    h = SCREEN_H;
    for (i = 0; i <= STEPS; i++)
      {
	if (keypressed ())
	  break;
	w = ((STEPS - i) * SCREEN_W) / STEPS;
	x = (SCREEN_W - w) / 2;
	aa_stretch_blit (bmp, screen, 0, 0, bmp->w, bmp->h, x, y, w, h);
      }
    for (i = 0; i <= STEPS; i++)
      {
	if (keypressed ())
	  break;
	w = ((STEPS - i) * SCREEN_W) / STEPS;
	h = ((STEPS - i) * SCREEN_H) / STEPS;
	x = (SCREEN_W - w) / 2;
	y = (SCREEN_H - h) / 2;
	aa_stretch_blit (bmp, screen, 0, 0, bmp->w, bmp->h, x, y, w, h);
      }
    readesc ();

    x = 0;
    w = bmp->w;
    for (i = 0; i <= STEPS; i++)
      {
	if (keypressed ())
	  break;
	h = (i * bmp->h) / STEPS;
	y = (bmp->h - h) / 2;
	aa_stretch_blit (bmp, screen, x, y, w, h, 0, 0, SCREEN_W, SCREEN_H);
      }
    y = 0;
    h = bmp->h;
    for (i = 0; i <= STEPS; i++)
      {
	if (keypressed ())
	  break;
	w = (i * bmp->w) / STEPS;
	x = (bmp->w - w) / 2;
	aa_stretch_blit (bmp, screen, x, y, w, h, 0, 0, SCREEN_W, SCREEN_H);
      }
    for (i = 0; i <= STEPS; i++)
      {
	if (keypressed ())
	  break;
	w = (i * bmp->w) / STEPS;
	h = (i * bmp->h) / STEPS;
	x = (bmp->w - w) / 2;
	y = (bmp->h - h) / 2;
	aa_stretch_blit (bmp, screen, x, y, w, h, 0, 0, SCREEN_W, SCREEN_H);
      }
    readesc ();
  }

  destroy_bitmap (bmp);
  allegro_exit ();

  return 0;
}

/*
 * strtest.c ends here
 */
